package lab04;

import java.io.Reader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;

import java.util.Stack;


// Balance class: check for balanced symbols
//
// CONSTRUCTION: with a Reader object
// ******************PUBLIC OPERATIONS***********************
// int checkBalance( )   --> Print mismatches
//                           return number of errors
// ******************ERRORS**********************************
// Error checking on comments and quotes is performed
// main checks for balanced symbols.


/**
 * Class to perform symbol matching
 * generation for Java programs.
 */
public class BalanceParenthesis
{
    /**
     * Symbol represents what will be placed on the stack.
     */
    private static class Symbol
    {
        public char  token;
        public int   theLine;

        public Symbol( char tok, int line )
        {
            token   = tok;
            theLine = line;
        }
    }
    
    /**
     * Constructor.
     * @param inStream the stream containing a program.
     */
    public BalanceParenthesis( Reader inStream )
    {
        errors = 0;
        tok = new Tokenizer( inStream );
    }


    /**
     * Print an error message for unbalanced symbols.
     * @return number of errors detected.
     */
    public int checkBalance( )
    {
        int errors = 0;

        /*** ??? ***/
        
        return errors;
    }

    private Tokenizer tok;
    private int errors;

    /**
     * Print an error message if clSym does not match opSym.
     * Update errors.
     */
    private void checkMatch( Symbol opSym, Symbol clSym )
    {
        if( opSym.token == '(' && clSym.token != ')' ||
            opSym.token == '[' && clSym.token != ']' ||
            opSym.token == '{' && clSym.token != '}' )
        {
            System.out.println( "Found " + clSym.token + " on line " +
                  tok.getLineNumber( ) + "; does not match " + opSym.token
                  + " at line " + opSym.theLine );
            errors++;
        }
    }

    /**
     * main routine for balanced symbol checker.
     * Slightly different from text.
     * If no command line parameters, standard input is used.
     * Otherwise, files in command line are used.
     */
    public static void main( String [ ] args )
    {
    	//Type Ctrl+Z on Windows to check remaining characters
    	//Type Ctrl+D on Mac, Linux, Unix to check remaining characters

    	BalanceParenthesis p;

        if( args.length == 0 )
        {
            p = new BalanceParenthesis( new InputStreamReader( System.in ) );
            if( p.checkBalance( ) == 0 )
                System.out.println( "No errors!" );
            return;
        }

        for( int i = 0; i < args.length; i++ )
        {
            FileReader f = null;
            try 
            {
                f = new FileReader( args[ i ] );

                System.out.println( args[ i ] + ": " );
                p = new BalanceParenthesis( f );
                if( p.checkBalance( ) == 0 ) 
                    System.out.println( "   ...no errors!" );
            }
            catch( IOException e )
            {
                System.err.println( e + args[ i ] );
            }
            finally
            {
                try 
                {
                    if( f != null )
                        f.close( );
                }
                catch( IOException e )
                {
                }
            }

        }
    }
}

